/*
 * Copyright 2018 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.google.android.overtheair.v1;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import com.google.api.pathtemplate.PathTemplate;
import com.google.api.resourcenames.ResourceName;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;

// AUTO-GENERATED DOCUMENTATION AND CLASS
@javax.annotation.Generated("by GAPIC protoc plugin")
public class ConfigName implements ResourceName {

  private static final PathTemplate PATH_TEMPLATE =
      PathTemplate.createWithoutUrlEncoding("deployments/{deployment}/configs/{config}");

  private volatile Map<String, String> fieldValuesMap;

  private final String deployment;
  private final String config;

  public String getDeployment() {
    return deployment;
  }

  public String getConfig() {
    return config;
  }

  public static Builder newBuilder() {
    return new Builder();
  }

  public Builder toBuilder() {
    return new Builder(this);
  }

  private ConfigName(Builder builder) {
    deployment = Preconditions.checkNotNull(builder.getDeployment());
    config = Preconditions.checkNotNull(builder.getConfig());
  }

  public static ConfigName of(String deployment, String config) {
    return newBuilder()
      .setDeployment(deployment)
      .setConfig(config)
      .build();
  }

  public static String format(String deployment, String config) {
    return newBuilder()
      .setDeployment(deployment)
      .setConfig(config)
      .build()
      .toString();
  }

  public static ConfigName parse(String formattedString) {
    if (formattedString.isEmpty()) {
      return null;
    }
    Map<String, String> matchMap =
        PATH_TEMPLATE.validatedMatch(formattedString, "ConfigName.parse: formattedString not in valid format");
    return of(matchMap.get("deployment"), matchMap.get("config"));
  }

  public static List<ConfigName> parseList(List<String> formattedStrings) {
    List<ConfigName> list = new ArrayList<>(formattedStrings.size());
    for (String formattedString : formattedStrings) {
      list.add(parse(formattedString));
    }
    return list;
  }

  public static List<String> toStringList(List<ConfigName> values) {
    List<String> list = new ArrayList<String>(values.size());
    for (ConfigName value : values) {
      if (value == null) {
        list.add("");
      } else {
        list.add(value.toString());
      }
    }
    return list;
  }

  public static boolean isParsableFrom(String formattedString) {
    return PATH_TEMPLATE.matches(formattedString);
  }

  public Map<String, String> getFieldValuesMap() {
    if (fieldValuesMap == null) {
      synchronized (this) {
        if (fieldValuesMap == null) {
          ImmutableMap.Builder<String, String> fieldMapBuilder = ImmutableMap.builder();
          fieldMapBuilder.put("deployment", deployment);
          fieldMapBuilder.put("config", config);
          fieldValuesMap = fieldMapBuilder.build();
        }
      }
    }
    return fieldValuesMap;
  }

  public String getFieldValue(String fieldName) {
    return getFieldValuesMap().get(fieldName);
  }

  @Override
  public String toString() {
    return PATH_TEMPLATE.instantiate("deployment", deployment, "config", config);
  }

  /** Builder for ConfigName. */
  public static class Builder {

    private String deployment;
    private String config;

    public String getDeployment() {
      return deployment;
    }

    public String getConfig() {
      return config;
    }

    public Builder setDeployment(String deployment) {
      this.deployment = deployment;
      return this;
    }

    public Builder setConfig(String config) {
      this.config = config;
      return this;
    }

    private Builder() {
    }

    private Builder(ConfigName configName) {
      deployment = configName.deployment;
      config = configName.config;
    }

    public ConfigName build() {
      return new ConfigName(this);
    }
  }

  @Override
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (o instanceof ConfigName) {
      ConfigName that = (ConfigName) o;
      return (this.deployment.equals(that.deployment))
          && (this.config.equals(that.config));
    }
    return false;
  }

  @Override
  public int hashCode() {
    int h = 1;
    h *= 1000003;
    h ^= deployment.hashCode();
    h *= 1000003;
    h ^= config.hashCode();
    return h;
  }
}

